// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.


#include <stdio.h>
#include <stdint.h>
#include <math.h>
#include <iostream>
#include <string>
#include <vector>
#include <utility>
#include "./McCaskillAlgorithm.h"



using std::cout;
using std::endl;



#define a -1
#define b -1
#define c -1
#define INF 100
#define BETA 0.00000000000000000000414
#define e 2.718

// Initialize the Q_matrices.
void McCaskillAlgorithm::initAlgorithm(const std::string& input)
{        _seqString = input;
        // Creating Matrix Q.
         _Q.resize(_seqString.length());
        // what is static_cast
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
         _Q[i].resize(_seqString.length());
        }

        // initializing the lower part of matrix Q with 1
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
                for (int j = 0; j <= i; ++j)
                    {
                        _Q[i][j] = 1;
                }
        }
        /*****************************************************************/

        // Creating Matrix Qb.
        _Qb.resize(_seqString.length());

        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
                _Qb[i].resize(_seqString.length());
        }

        // Initializing the lower part of matrix Qb with 0

        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
               for (int j = 0; j <= i; ++j)
                {
                         _Qb[i][j] = 0;
                }
        }
        /******************************************************************/

        // Creating Matrix Qm.
        _Qm.resize(_seqString.length());

        // what is static_cast
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
                _Qm[i].resize(_seqString.length());
        }

        // initializing the lower part of matrix Q with 1
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
                for (int j = 0; j <= i; ++j)
                {
                        _Qm[i][j] = 0;
                }
        }
        /*****************************************************************/

        // Creating Matrix Qm1.
        _Qm1.resize(_seqString.length());

        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
                _Qm1[i].resize(_seqString.length());
        }
        // Initializing the lower part of matrix Qb with 0

        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {
               for (int j = 0; j <= i; ++j)
                {
                         _Qm1[i][j] = 0;
                }
        }
}





// Calculate the values of entries of matrix _Q
void McCaskillAlgorithm::computeQ()
{       int i, j, k;
        float val, sum = 0;
        for (int d = 1; d < static_cast<int>(_seqString.length()); d++)
        {
                for (i = 0; i < static_cast<int>(_seqString.length()); i++)
                {
                        j = i + d;
                        if (j < static_cast<int>(_seqString.length()))
                           {for (k = i + 1; k < j; k++)
                                {       val = _Q[i][k-1] + computeQb(k, j);
                                        sum = sum + val;
                                }
                                _Q[i][j] = _Q[i][j-1] +  sum;
                        }
                 }
        }
}



// Calculate the minimum energy of matrix _Qb.
float McCaskillAlgorithm::computeQb(int i, int j)
{       float eH = _eH(_seqString.at(i), _seqString.at(j));
        float sum1, sum2, sum3 = 0;

        sum1 = pow(e , (-1 * BETA * eH));

        for (int ip = i + 1; ip < j - 1; ip++)
            {
                for (int jp = ip + 1; jp < j ; jp++)
{sum2 = sum2+pow(e, -1*_eSBI(i, j, ip, jp*BETA*computeQb(ip, jp)));}
        }
        for (int k = i + 1; k < j - 2; k++)
                 {
sum3 = sum3+pow(e, -1*BETA*a)* computeQm(i+1, k-1) *computeQm1(k, j-1);
}

        _Qb[i][j] = sum1+sum2+sum3;

        return _Qb[i][j];
}


// Calculate the minimum energy of matrix _Qm.
float McCaskillAlgorithm::computeQm(int i, int j)
{       float sum = 0;
        for (int k = i ; k < j -1; k++)
            {
                 sum = sum + (pow(e, -1 * BETA * (k - i) * c)
                       + computeQm(i, k-1)) * computeQm1(k, j);
}

        _Qm[i][j] = sum;
        return _Qm[i][j];
}

// Calculate the minimum energy of matrix _Qm1.
float McCaskillAlgorithm::computeQm1(int i, int j)
{       float sum = 0;
        for (int k = i+2 ; k <= j ; k++)
{sum = sum+pow(e, -1*BETA*b)*computeQb(i, k)* pow(e, -1 * BETA  * (j - k)*c);
}

        _Qm1[i][j] = sum;
        return _Qm1[i][j];
}


// Method for calculating the energies of H.
float McCaskillAlgorithm::_eH(char x, char y)
{
  if ((x == 'G' && y == 'C') || (x == 'C' && y == 'G') ||
      (x == 'A' && y == 'U') || (x == 'U' && y == 'A'))
     return -1;
  else
    return INF;
}


// Method for calculating the energies of multi and interior loop.
float McCaskillAlgorithm::_eSBI(char i, char j, char x, char y)
{     if (      (i == 'A' && j == 'U' && x == 'G' && y == 'C') ||
                (i == 'U' && j == 'A' && x == 'C' && y == 'G') ||
                (i == 'A' && j == 'U' && x == 'C' && y == 'G') ||
                (i == 'U' && j == 'A' && x == 'G' && y == 'C') ||
                (i == 'A' && j == 'U' && x == 'A' && y == 'U') ||
                (i == 'A' && j == 'U' && x == 'U' && y == 'A') ||
                (i == 'U' && j == 'A' && x == 'A' && y == 'U') ||
                (i == 'U' && j == 'A' && x == 'U' && y == 'A') ||
                (i == 'C' && j == 'G' && x == 'C' && y == 'G') ||
                (i == 'C' && j == 'G' && x == 'G' && y == 'C') ||
                (i == 'G' && j == 'C' && x == 'C' && y == 'G') ||
                (i == 'G' && j == 'C' && x == 'G' && y == 'C') ||
                (i == 'C' && j == 'G' && x == 'A' && y == 'U') ||
                (i == 'C' && j == 'G' && x == 'U' && y == 'A') ||
                (i == 'G' && j == 'C' && x == 'A' && y == 'U') ||
                (i == 'G' && j == 'C' && x == 'U' && y == 'A'))
          return -1;
          else
            return INF;
}



// Compute the structure of the input sequence.
void McCaskillAlgorithm::computeStructure(const std::string& input,
std::vector<std::pair<int, int> >* result)
{
          initAlgorithm(input);
          computeQ();
}


// ___________________________________________________________________________
// Print the matrices.
void McCaskillAlgorithm::printMatrix() const
{       printf("\nTHIS IS MATRIX  _Q:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
            {
                for (size_t j = 0; j < _seqString.length(); ++j)
                    {
                        cout << " " << _Q[i][j];
                }
                cout << endl;
        }
        printf("\nTHIS IS MATRIX _Qb:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
            {
                for (size_t j = 0; j < _seqString.length(); ++j)
                    {
                         cout << " " << _Qb[i][j];
                }
                cout << endl;
        }
        printf("\nTHIS IS MATRIX  _Qm:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
            {
                for (size_t j = 0; j < _seqString.length(); ++j)
                    {
                        cout << " " << _Qm[i][j];
                }
                cout << endl;
        }
        printf("\nTHIS IS MATRIX _Qm1:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
            {
                for (size_t j = 0; j < _seqString.length(); ++j)
                {
                         cout << " " << _Qm1[i][j];
                }
                cout << endl;
        }
}

float McCaskillAlgorithm::getPartitionFunction()
{
        return _Q[0][_seqString.length()-1];
}

void  McCaskillAlgorithm::convertStructureString(string str_string,
std::vector<std::pair<int, int> > &structure)
{
        while (str_string != "")
        {
                string pair = str_string.substr(0, str_string.find(')')+1);
                str_string = str_string.substr(str_string.find(')')+1);

                string first = pair.substr(pair.find('(')+1, pair.find(',')-1);
                string second = pair.substr(pair.find(',')+1, pair.find(')')-3);

                structure.push_back(std::make_pair(atoi(first.c_str())-1,
atoi(second.c_str())-1));
        }
}

float McCaskillAlgorithm::getLoopEnergy(const std::vector<std::pair<int,
      int> > structure, unsigned p_if, unsigned p_is)
{
        float energy = 0;

        int count = 0;
        std:vector<std::pair<int, int> > inner_pairs;
        for (size_t k = 0; k < structure.size(); k++)
        {
                if (structure[k].first > p_if && structure[k].second < p_is)
                {
                        energy += getLoopEnergy(structure, structure[k].first,
structure[k].second);
                        count++;
                        inner_pairs.push_back(std::make_pair(structure[k].
first, structure[k].second));
                }
        }

        if (count == 0)
        return energy+_eH (_seqString[p_if], _seqString[p_is]);
        else if (count == 1)
        {
         return energy+_eSBI (_seqString[p_if], _seqString[p_is],
_seqString[inner_pairs[0].first], _seqString[inner_pairs[0].second]);
        }
        else
        {
                int num_unpaired = 0;

                num_unpaired += inner_pairs[0].first - p_if -1;
                for (size_t i = 1; i+1 < inner_pairs.size();i++)
                {
                 num_unpaired += inner_pairs[i+1].first -
                 inner_pairs[i].second -1;
                }

                return energy + a + b*count + c*num_unpaired;
        }

        return energy;
}

float McCaskillAlgorithm::getEnergy(const std::vector<std::pair<int, int> >
      structure)
{
        float energy = 0;

        for (size_t i = 0; i < _seqString.size(); i++)
        {
                 for (size_t j = 0; j < structure.size(); j++)
                 {
                        if (i == structure[j].first)
                        {
                                unsigned p_if = i;
                                unsigned p_is = structure[j].second;

                                energy += getLoopEnergy(structure, p_if, p_is);

                                i = structure[j].second;
                        }
                 }
        }

        printf("energy : %f\n", energy);
        return pow(e, -1*BETA*energy);
}
