// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.

#ifndef MCCASKILLALGORITHM_MCCASKILLALGORITHM_H_
#define MCCASKILLALGORITHM_MCCASKILLALGORITHM_H_



#include <gtest/gtest.h>
#include <sys/time.h>
#include <string>
#include <vector>
#include <utility>



using std::string;
using std::vector;

class McCaskillAlgorithm
{
    public:
        // compute the structure .
        void computeStructure(const std::string& input,
std::vector<std::pair<int, int> >* result);

        // Initialize the McCaskill Matrix.
        FRIEND_TEST(McCaskillAlgorithmTest, initAlgorithm);
        void initAlgorithm(const std::string& input);

        // Print the Matrices.
        void printMatrix() const;

        // Print the corresponding structure.
        void printStructure
        (const std::vector<std::pair<int, int> >&result) const;

        float getPartitionFunction();

        void convertStructureString(string str_string,
          std::vector<std::pair<int, int> > &structure);

        float getEnergy(const std::vector<std::pair<int, int> > structure);

        float getLoopEnergy(const std::vector<std::pair<int, int> > structure,
              unsigned p_if, unsigned p_is);
    private:
        // Method to calculate the minimum energy of the given string.
        FRIEND_TEST(McCaskillAlgorithmTest, computeQ);
        void computeQ();

        // Calculate the values of Qb.
        FRIEND_TEST(McCaskillAlgorithmTest, computeQb);
        float computeQb(int, int);

       // Calculate the values of Qb.
        FRIEND_TEST(McCaskillAlgorithmTest, computeQm);
        float computeQm(int, int);

       // Calculate the values of Qb.
        FRIEND_TEST(McCaskillAlgorithmTest, computeQm1);
        float computeQm1(int, int);

       // energies
        FRIEND_TEST(McCaskillAlgorithmTest, _eH);
        float _eH(char, char);
        FRIEND_TEST(McCaskillAlgorithmTest, _eSBI);
        float _eSBI(char i, char j, char x, char y);

        // Declare the matrices and the constants.
        string _seqString;
        vector< vector<float> > _Q;
        vector< vector<float> > _Qb;
        vector< vector<float> > _Qm;
        vector< vector<float> > _Qm1;
};


#endif  // MCCASKILLALGORITHM_MCCASKILLALGORITHM_H_
