// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include <utility>
#include "./McCaskillAlgorithm.h"

// Test for the method initZukerAlgorithm.
TEST(McCaskillAlgorithmTest, initAlgorithm)
{
  McCaskillAlgorithm m;

  // Test the string "AAA".
  m.initAlgorithm("CGAU");
  ASSERT_EQ(m._seqString, "CGAU");
  ASSERT_EQ(m._Q.size(), 4);
  ASSERT_EQ(m._Q[0].size(), 4);
  ASSERT_EQ(m._Q[1].size(), 4);
  ASSERT_EQ(m._Q[2].size(), 4);
  ASSERT_EQ(m._Q[3].size(), 4);

  ASSERT_EQ(m._Qb.size(), 4);
  ASSERT_EQ(m._Qb[0].size(), 4);
  ASSERT_EQ(m._Qb[1].size(), 4);
  ASSERT_EQ(m._Qb[2].size(), 4);
  ASSERT_EQ(m._Qb[3].size(), 4);

  ASSERT_EQ(m._Q[0][0], 1);
  ASSERT_EQ(m._Q[1][1], 1);
  ASSERT_EQ(m._Q[2][1], 1);
  ASSERT_EQ(m._Q[2][2], 1);
  ASSERT_EQ(m._Q[1][0], 1);
  ASSERT_EQ(m._Q[2][0], 1);
  ASSERT_EQ(m._Q[2][1], 1);
  ASSERT_EQ(m._Qb[1][1], 0);
  ASSERT_EQ(m._Qb[2][2], 0);
  ASSERT_EQ(m._Qb[1][0], 0);
  ASSERT_EQ(m._Qb[2][1], 0);
  ASSERT_EQ(m._Qb[2][0], 0);
  ASSERT_EQ(m._Qb[2][1], 0);

  // Test the empty string.
  m.initAlgorithm("");
  ASSERT_EQ(m._seqString, "");
  ASSERT_EQ(m._Q.size(), 0);
  ASSERT_EQ(m._Qb.size(), 0);

  // Test the string GAUAU.
  m.initAlgorithm("GAUAU");
  ASSERT_EQ(m._seqString, "GAUAU");
  ASSERT_EQ(m._Q.size(), 5);
  ASSERT_EQ(m._Q[0].size(), 5);
  ASSERT_EQ(m._Q[1].size(), 5);
  ASSERT_EQ(m._Q[2].size(), 5);
  ASSERT_EQ(m._Q[3].size(), 5);
  ASSERT_EQ(m._Q[4].size(), 5);

  ASSERT_EQ(m._Qb.size(), 5);
  ASSERT_EQ(m._Qb[0].size(), 5);
  ASSERT_EQ(m._Qb[1].size(), 5);
  ASSERT_EQ(m._Qb[2].size(), 5);
  ASSERT_EQ(m._Qb[3].size(), 5);
  ASSERT_EQ(m._Qb[4].size(), 5);

  ASSERT_EQ(m._Q[0][0], 1);
  ASSERT_EQ(m._Q[1][1], 1);
  ASSERT_EQ(m._Q[2][2], 1);
  ASSERT_EQ(m._Q[3][3], 1);
  ASSERT_EQ(m._Q[4][4], 1);

  ASSERT_EQ(m._Q[1][0], 1);
  ASSERT_EQ(m._Q[2][0], 1);
  ASSERT_EQ(m._Q[3][0], 1);
  ASSERT_EQ(m._Q[4][0], 1);
  ASSERT_EQ(m._Q[2][1], 1);
  ASSERT_EQ(m._Q[3][1], 1);
  ASSERT_EQ(m._Q[3][2], 1);
  ASSERT_EQ(m._Q[4][1], 1);
  ASSERT_EQ(m._Q[4][2], 1);
  ASSERT_EQ(m._Q[4][3], 1);

  ASSERT_EQ(m._Qb[0][0], 0);
  ASSERT_EQ(m._Qb[1][1], 0);
  ASSERT_EQ(m._Qb[2][2], 0);
  ASSERT_EQ(m._Qb[3][3], 0);
  ASSERT_EQ(m._Qb[4][4], 0);

  ASSERT_EQ(m._Qb[1][0], 0);
  ASSERT_EQ(m._Qb[2][0], 0);
  ASSERT_EQ(m._Qb[3][0], 0);
  ASSERT_EQ(m._Qb[4][0], 0);
  ASSERT_EQ(m._Qb[2][1], 0);
  ASSERT_EQ(m._Qb[3][1], 0);
  ASSERT_EQ(m._Qb[3][2], 0);
  ASSERT_EQ(m._Qb[4][1], 0);
  ASSERT_EQ(m._Qb[4][2], 0);
  ASSERT_EQ(m._Qb[4][3], 0);
}

