// Copyright 2010, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.

#include <string>
#include <vector>
#include <utility>
#include "./stdio.h"
#include "./NussinovAlgorithm.h"

using std::string;
using std::vector;
using std::pair;
using std::make_pair;



// _____________________________________________________________________________

NussinovAlgorithm::NussinovAlgorithm()
{
}
//

void NussinovAlgorithm::computeStructure(
    const string& seq, vector<pair<int, int> >* pairs)
{
// For now, do nothing and just return the pair (0,0).
  _inputSequence = seq;  // transfer the _inputSequence directly later;
  for (int i = 0; i < _inputSequence.size(); i++)
      _nussinovMatrix.push_back(vector<int>());
  for (int i = 0; i < _inputSequence.size(); i++)
      for (int j = 0; j < _inputSequence.size(); j++)
      _nussinovMatrix[i].push_back(0);
  // tell the memory prepare a space for saving the _nussinovMatrix result;

  for (int i = 0; i < _inputSequence.size(); i++)
      for (int j = 0; j < _inputSequence.size(); j++)
      {
      if (i == j || j == i-1)
        _nussinovMatrix[i][j] = 0;
      else
        _nussinovMatrix[i][j] = -1;
      }
  // according to the NussinovAlgorithm and give value -1
  // when we don't need to compute it in the _nussinovMatrix
pairs->clear();
computeNussinovMatrix(0, _inputSequence.size()-1);
tracebackNussinovMatrix(0, _inputSequence.size()-1, pairs);
}

void NussinovAlgorithm::printNussinovMatrix()  // print out the Nussinov Matrix
{
printf("The Nussinove _nussinovMatrix is: \n");
  // pairs->push_back(std::make_pair(0, 0));
  for (int i = 0; i < _inputSequence.size(); i++)
  {
      for (int j = 0; j < _inputSequence.size(); j++)
       printf("%d\t", _nussinovMatrix[i][j]);
       printf("\n");
  }
  // output the _nussinovMatrix;
}

int NussinovAlgorithm::computeNussinovMatrix(int i, int j)
// compute the Nussinov Matrix
{
if (i > j + 1)
{
      printf("error occurs!\n");
      return -1;
}
if (i == j || j == i-1)
      return 0;
  int _maxValue = 0;
  int _oldValue = computeNussinovMatrix(i, j-1);
  _maxValue = _oldValue;
  // compute NussinovMatrix and put the maxValue in the NussinovMatrix;
    for (int k = i; k < j; k++)
        {
      if ((_inputSequence[k] == 'A' && _inputSequence[j] == 'U') ||
         (_inputSequence[k] == 'U' && _inputSequence[j] == 'A') ||
         (_inputSequence[k] == 'C' && _inputSequence[j] == 'G') ||
         (_inputSequence[k] == 'G' && _inputSequence[j] == 'C'))
      {}
      // definition of base pairs;
      else
          continue;
      int n1, n2;
      if (i == 0 && k == 0)
          n1 = 0;
      else
      {
          if (_nussinovMatrix[i][k-1] != -1)
              n1 = _nussinovMatrix[i][k-1];
          else
              n1 = computeNussinovMatrix(i, k-1);
      }

          if (j == 0)
             continue;
      if (_nussinovMatrix[k+1][j-1] != -1)
          n2 = _nussinovMatrix[k+1][j-1];
      else
          n2 = computeNussinovMatrix(k+1, j-1);
      int sub = n1 + n2 + 1;
      if (sub > _maxValue)
       _maxValue = sub;
       }
       _nussinovMatrix[i][j] = _maxValue;

       return _maxValue;
}

void NussinovAlgorithm::tracebackNussinovMatrix(int i, int j,
vector<std::pair<int, int> >* pairs)
{
      if (j <= i)
         return;
      else if (_nussinovMatrix[i][j] == _nussinovMatrix[i][j-1])
      {
      tracebackNussinovMatrix(i, j-1, pairs);
      return;
      }
      else
      {
      for (int k = i; k < j; k++)
      {
      if ((_inputSequence[k] == 'A' && _inputSequence[j] == 'U') ||
         (_inputSequence[k] == 'U' && _inputSequence[j] == 'A') ||
         (_inputSequence[k] == 'C' && _inputSequence[j] == 'G') ||
         (_inputSequence[k] == 'G' && _inputSequence[j] == 'C'))
         {}
      else
         continue;

      if (k == 0 && j != 0)
         {
         if (_nussinovMatrix[i][j] == _nussinovMatrix[k+1][j-1] + 1)
            {
               pairs->push_back(make_pair(k, j));
// save pair (k,j) as resulting pair
               tracebackNussinovMatrix(i, k-1, pairs);
// traceback the substring from i to k-1
               tracebackNussinovMatrix(k+1, j-1, pairs);
// traceback the substring from k+1 to j-1
               return;
              }
          }
          else if (k != 0 && j == 0)
          {
            if (_nussinovMatrix[i][j] == _nussinovMatrix[i][k-1] + 1)
             {
               pairs->push_back(make_pair(k, j));
               tracebackNussinovMatrix(i, k-1, pairs);
               tracebackNussinovMatrix(k+1, j-1, pairs);
               return;
              }
           }
           else if (k == 0 && j == 0)
           {
             if (_nussinovMatrix[i][j] == 1)
           {
               pairs->push_back(make_pair(k, j));
               tracebackNussinovMatrix(i, k-1, pairs);
               tracebackNussinovMatrix(k+1, j-1, pairs);
             return;
            }
           }
            else
         {
           if (_nussinovMatrix[i][j] ==
               _nussinovMatrix[i][k-1] + _nussinovMatrix[k+1][j-1] +1)
             {
               pairs->push_back(make_pair(k, j));
               tracebackNussinovMatrix(i, k-1, pairs);
               tracebackNussinovMatrix(k+1, j-1, pairs);
               return;
           }
         }
      }
    }
}
