// Copyright 2010, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gm// ail.com>.

#ifndef NUSSINOVALGORITHM_NUSSINOVALGORITHM_H_
#define NUSSINOVALGORITHM_NUSSINOVALGORITHM_H_

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include <utility>

using std::string;
using std::vector;
using std::pair;

// Class for computing the structure of a given RNA sequence with the maximal
// number of base pairs, using the Nussinov algorithm.
class NussinovAlgorithm
{
 public:
  NussinovAlgorithm();

  // For a given RNA sequence (just a string of A,C,G,U), compute the structure
  // with the maximal number of pairs.
  void computeStructure(const  std::string& seq,
                         std::vector< pair<int, int> >* pairs);

  void printNussinovMatrix();
  // The Nussinov matrix.
  vector<vector<int> > _nussinovMatrix;
 private:
  // Compute the Nussinov matrix.
  int computeNussinovMatrix(int i, int j);
  FRIEND_TEST(NussinovAlgorithmTest, computeNussinovMatrix);
  string _inputSequence;

  // Traceback in the Nussinov matrix to get the pairing with the maximal number
  // of pairs.
  void tracebackNussinovMatrix(int i, int j,
       vector<pair<int, int> >* pairs);
};


#endif  // NUSSINOVALGORITHM_NUSSINOVALGORITHM_H_
