// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.


#include <stdio.h>
#include <stdint.h>
#include <iostream>
#include <string>
#include <vector>
#include <utility>
#include "./ZukerAlgorithm.h"

using std::cout;
using std::endl;

#define a -1
#define INF 100

// Initialize the ZukerAlgorithm_matrices.
void ZukerAlgorithm::initZukerAlgorithm(const std::string& input)
{_seqString = input;
        // Creating Matrix W.
        _W.resize(_seqString.length());
        // what is static_cast.
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
{_W[i].resize(_seqString.length());
        }
        // initializing the lower part of matrix W with 0
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
{for (int j = 0; j <= i; ++j)
{                        _W[i][j] = 0;
                }
        }
        /***************************************************************/
        // Creating Matrix V.
        _V.resize(_seqString.length());
for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
{_V[i].resize(_seqString.length());
        }
        // Initializing the lower part of matrix V with infinity
        // where infinity = 100
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
            {for (int j = 0; j <= i; ++j)
                { _V[i][j] = INF;
                }
        }
        /******************************************************************/
        // Matrix _traceBackW.
        _traceBackW.resize(_seqString.length());

        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
            {_traceBackW[i].resize(_seqString.length());
        }

        // initializing the lower part of matrix traceBackW.
        // with 0.
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
{
             for (int j = 0; j < static_cast<int>(_seqString.length()); ++j)
        {
                        _traceBackW[i][j] = 0;
           }
         }
        /******************************************************************/

        // Matrix _traceBackV.
        _traceBackV.resize(_seqString.length());
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
        {_traceBackV[i].resize(_seqString.length());
        }

        // initializing the lower part of matrix traceBackV with 0
        for (int i = 0; i < static_cast<int>(_seqString.length()); ++i)
            {for (int j = 0; j < static_cast<int>(_seqString.length()); ++j)
                  {_traceBackV[i][j] = 0;
                }
        }
}

// Method for calculating the energies of Stacking loop.
float ZukerAlgorithm::_eS(char x, char y)
{  if ((x == 'G' && y == 'C') || (x == 'C' && y == 'G') ||
       (x == 'A' && y == 'U') || (x == 'U' && y == 'A'))
    return -1;
  else
    return INF;
}

// Method for calculating the energies of Hairpin loop.
float ZukerAlgorithm::_eH(char x, char y)
{  if ((x == 'G' && y == 'C') || (x == 'C' && y == 'G') ||
       (x == 'A' && y == 'U') || (x == 'U' && y == 'A'))
    return -1;
  else
    return INF;
}
// Method for calculating the energies of multi and interior loop.
float ZukerAlgorithm::_eL(char i, char j, char x, char y)
{     if (      (i == 'A' && j == 'U' && x == 'G' && y == 'C') ||
                (i == 'U' && j == 'A' && x == 'C' && y == 'G') ||
                (i == 'A' && j == 'U' && x == 'C' && y == 'G') ||
                (i == 'U' && j == 'A' && x == 'G' && y == 'C') ||
                (i == 'A' && j == 'U' && x == 'A' && y == 'U') ||
                (i == 'A' && j == 'U' && x == 'U' && y == 'A') ||
                (i == 'U' && j == 'A' && x == 'A' && y == 'U') ||
                (i == 'U' && j == 'A' && x == 'U' && y == 'A') ||
                (i == 'C' && j == 'G' && x == 'C' && y == 'G') ||
                (i == 'C' && j == 'G' && x == 'G' && y == 'C') ||
                (i == 'G' && j == 'C' && x == 'C' && y == 'G') ||
                (i == 'G' && j == 'C' && x == 'G' && y == 'C') ||
                (i == 'C' && j == 'G' && x == 'A' && y == 'U') ||
                (i == 'C' && j == 'G' && x == 'U' && y == 'A') ||
                (i == 'G' && j == 'C' && x == 'A' && y == 'U') ||
                (i == 'G' && j == 'C' && x == 'U' && y == 'A'))
          return -1;
          else
            return INF;
}

// Calculate the values of entries of matrix _W
void ZukerAlgorithm::computeW()
{       int i, j, k;
        float val, min = 0;
        for (int d = 1; d < static_cast<int>(_seqString.length()); d++)
             {for (i = 0; i < static_cast<int>(_seqString.length()); i++)
{ j = i + d;
                        if (j < static_cast<int>(_seqString.length()))
                               {min = 0;
                 // Case 1.
                                if (_W[i][j - 1] < min)
                                   {    min = _W[i][j - 1];
                                        _traceBackW[i][j] = -1;
                                }
                                if (_W[i + 1][j] < min)
                                   {    min = _W[i + 1][j];
                                        _traceBackW[i][j] = -2;
                                }
               // Case 2.
                                if (computeV(i, j) < min)
                                   {    min = computeV(i, j);
                                        _traceBackW[i][j] = -3;
                                }
               // Case 3.
                                for (k = i + 1; k < j; k++)
                                   {    val = _W[i][k] + _W[k + 1][j];
                                        if (val < min)
                                        {       min = val;
                                                _traceBackW[i][j] = k;
                                        }
                                }
                                _W[i][j] = min;
                        }
                 }
        }
}



// Calculate the minimum energy of matrix _V.
float ZukerAlgorithm::computeV(int i, int j)
{     float min = 0, tmp = 0;
  // Case 1.
        if (_eH(_seqString.at(i), _seqString.at(j)) < min)
          {     min = _eH(_seqString.at(i), _seqString.at(j));
                _traceBackV[i][j] = -1;
          }
  // Case 2.
       if (_eS(_seqString.at(i), _seqString.at(j)) + _V[i + 1][j - 1] < min)
           { min = _eS(_seqString.at(i), _seqString.at(j)) + _V[i + 1][j - 1];
            _traceBackV[i][j] = -2;
        }
  // Case 3.
        for (int k = i + 1; k < j - 1; ++k)
        {       tmp = _W[i + 1][k] + _W[k + 1][j - 1] + a;
                   if (tmp < min)
                 {     min = tmp;
                       _traceBackV[i][j] = k;
                }
        }
  // Case 4.
        for (int k = i + 1; k < j - 2; ++k)
             {for (int l = i + 2; l < j - 1; ++l)
               {if ((k - i + j - l) > 2)
                  tmp = _V[k][l] + _eL(_seqString.at(i), _seqString.at(j),
                        _seqString.at(k), _seqString.at(l));
                           if (tmp < min)
                              { min = tmp;
                                _traceBackV[i][j] = -3;
                        }
               }
        }
        _V[i][j] = min;
        return _V[i][j];
}



// Traceback to get the corresponding best structure.
void ZukerAlgorithm::traceBackW
(int i, int j, std::vector<std::pair<int, int> >* result)
        { if (_traceBackW[i][j] == 0)
          {return;
        } else if (_traceBackW[i][j] == -1)
          {traceBackW(i, j - 1, result);
        } else if (_traceBackW[i][j] == -2)
          {traceBackW(i + 1, j, result);
        } else if (_traceBackW[i][j] == -3)
          {traceBackV(i, j, result);
        } else
          {     int k = _traceBackW[i][j];
                if ((i + 1 <= k) &&(k + 1 <= j))
                   {    traceBackW(i, k, result);
                        traceBackW(k + 1, j, result);
                }
        }
}

// Traceback to get the corresponding best structure.
void ZukerAlgorithm::traceBackV
(int i, int j, std::vector<std::pair<int, int> >* result)
        { if (_traceBackV[i][j] == 0)
               {return;
        } else if (_traceBackV[i][j] == -1)
               {result -> push_back(std::make_pair(i, j));
        } else if (_traceBackV[i][j] == -2)
               {traceBackV(i + 1, j - 1, result);
                result -> push_back(std::make_pair(i, j));
        } else if (_traceBackV[i][j] == -3)
                {result -> push_back(std::make_pair(i, j));
                traceBackV(i + 1, j - 1, result);
            } else
              { int k = _traceBackV[i][j];
                if ((i + 1 <= k) && (k + 1 <= j - 1))
                {traceBackW(i + 1, k, result);
                 traceBackW(k + 1, j - 1, result);
            }
         }
}



// Compute the structure of the input sequence.
void ZukerAlgorithm::computeStructure
(const std::string& input, std::vector<std::pair<int, int> >* result)
{         initZukerAlgorithm(input);
          computeW();
          traceBackW(0, _seqString.length() - 1, result);
}



// ___________________________________________________________________________
// Print the matrices.
void ZukerAlgorithm::printMatrix() const
{  printf("\nTHIS IS MATRIX  _W:\n");
          for (size_t i = 0; i < _seqString.length(); ++i)
              {for (size_t j = 0; j < _seqString.length(); ++j)
                {cout << " " << _W[i][j];
                }
                cout << endl;
        }
        printf("\nTHIS IS MATRIX _V:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
               {for (size_t j = 0; j < _seqString.length(); ++j)
               {cout << " " << _V[i][j];
                }
                cout << endl;
        }

        printf("\nTHIS IS MATRIX _TRACEBACKW:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
                 {for (size_t j = 0; j < _seqString.length(); ++j)
                 {cout << " " << _traceBackW[i][j];
                }
                cout << endl;
        }

        printf("\nTHIS IS MATRIX _TRACEBACKV:\n");
        for (size_t i = 0; i < _seqString.length(); ++i)
             {for (size_t j = 0; j < _seqString.length(); ++j)
               {cout << " " << _traceBackV[i][j];
         }
                cout << endl;
       }
}



// Output the corresponding structure of the given string.
void ZukerAlgorithm::printStructure(const std::vector < std::
pair<int, int> >& result) const
{printf("The CORRESPONDING RNA STRUCTURE IS:\n");
        for (size_t i = 0; i < result.size(); i++)
        {printf("(%u,%u)", result[i].first, result[i].second);
        }
        printf("\n");
}
