// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.



#ifndef ZUKERALGORITHM_ZUKERALGORITHM_H_
#define ZUKERALGORITHM_ZUKERALGORITHM_H_

#include <gtest/gtest.h>
#include <sys/time.h>
#include <string>
#include <vector>
#include <utility>

using std::string;
using std::vector;

class ZukerAlgorithm
{
    public:
        // compute the structure .
        void computeStructure(const std::string& input,
        std::vector<std::pair<int, int> >* result);

        // Initialize the zuker algorithm.
        FRIEND_TEST(ZukerAlgorithmTest, initZukerAlgorithm);
        void initZukerAlgorithm(const std::string& input);

        // Print the Matrices.
        void printMatrix() const;

        // Print the corresponding structure.
        void printStructure(const std::vector<std::pair<int, int> >&result)
        const;

    private:
        // Energy function for Stacking loop.
        FRIEND_TEST(ZukerAlgorithmTest, _eS);
        float _eS(char x, char y);

        // Energy function for Hairpin loop.
        FRIEND_TEST(ZukerAlgorithmTest, _eH);
        float _eH(char x, char y);

        // Energy function for Multi und Interior loop.
        FRIEND_TEST(ZukerAlgorithmTest, _eL);
        float _eL(char i, char j, char iP, char jP);


        // trace back the matrix W to get the best structure.
        void traceBackW(int i, int j, std::vector
             <std::pair<int, int> >* result);

        // Method for tracing back the matrix V.
        void traceBackV(int, int, std::vector<std::pair<int, int> >* result);

        // Method to calculate the minimum energy of the given string.
        FRIEND_TEST(ZukerAlgorithmTest, computeW);
        void computeW();

        // Calculate the values of v.
        FRIEND_TEST(ZukerAlgorithmTest, computeV);
        float computeV(int, int);

        // Declare the matrices and the constants.
        string _seqString;
        vector< vector<float> > _W;
        vector< vector<float> > _V;
        vector< vector<float> > _traceBackW;
        vector< vector<float> > _traceBackV;
        float a;
};

#endif  // ZUKERALGORITHM_ZUKERALGORITHM_H_
