// Copyright 2011, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Author: Tuti Andriani <my.verdantique@gmail.com>,
// Zhilu Chang <zhilu.chang@gmail.com>.

#include <gtest/gtest.h>
#include <string>
#include <vector>
#include <utility>
#include "./ZukerAlgorithm.h"


// Test for the Method eS.

TEST(ZukerAlgorithmTest, _eS)
{
  ZukerAlgorithm z;

  std::vector<float> result_eS;
  result_eS.push_back(z._eS('G', 'C'));
  result_eS.push_back(z._eS('A', 'U'));
  result_eS.push_back(z._eS('C', 'G'));
  result_eS.push_back(z._eS('U', 'A'));
  result_eS.push_back(z._eS('A', 'G'));

  ASSERT_FLOAT_EQ(result_eS[0], -1);
  ASSERT_FLOAT_EQ(result_eS[1], -1);
  ASSERT_FLOAT_EQ(result_eS[2], -1);
  ASSERT_FLOAT_EQ(result_eS[3], -1);
  ASSERT_FLOAT_EQ(result_eS[4], 100);
}


// Test for the Method eH.

TEST(ZukerAlgorithmTest, _eH)
{
  ZukerAlgorithm z;
  std::vector<float> result_eH;
  result_eH.push_back(z._eH('G', 'C'));
  result_eH.push_back(z._eH('A', 'U'));
  result_eH.push_back(z._eH('C', 'G'));
  result_eH.push_back(z._eH('U', 'A'));
  result_eH.push_back(z._eH('A', 'G'));

  ASSERT_FLOAT_EQ(result_eH[0], -1);
  ASSERT_FLOAT_EQ(result_eH[1], -1);
  ASSERT_FLOAT_EQ(result_eH[2], -1);
  ASSERT_FLOAT_EQ(result_eH[3], -1);
  ASSERT_FLOAT_EQ(result_eH[4], 100);
}


// Test for the method initZukerAlgorithm.
TEST(ZukerAlgorithmTest, initZukerAlgorithm)
{
  ZukerAlgorithm z;

  // Test the string "AAA".
  z.initZukerAlgorithm("AAA");
  ASSERT_EQ(z._seqString, "AAA");
  ASSERT_EQ(z._W.size(), 3);
  ASSERT_EQ(z._W[0].size(), 3);
  ASSERT_EQ(z._W[1].size(), 3);
  ASSERT_EQ(z._W[2].size(), 3);
  ASSERT_EQ(z._V.size(), 3);
  ASSERT_EQ(z._V[0].size(), 3);
  ASSERT_EQ(z._V[1].size(), 3);
  ASSERT_EQ(z._V[2].size(), 3);
  ASSERT_EQ(z._W[0][0], 0);
  ASSERT_EQ(z._W[1][1], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._W[2][2], 0);
  ASSERT_EQ(z._W[1][0], 0);
  ASSERT_EQ(z._W[2][0], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._V[1][1], 100);
  ASSERT_EQ(z._V[2][2], 100);
  ASSERT_EQ(z._V[1][0], 100);
  ASSERT_EQ(z._V[2][1], 100);
  ASSERT_EQ(z._V[2][0], 100);
  ASSERT_EQ(z._V[2][1], 100);

  // Test the empty string.
  z.initZukerAlgorithm("");
  ASSERT_EQ(z._seqString, "");
  ASSERT_EQ(z._W.size(), 0);
  ASSERT_EQ(z._V.size(), 0);

  // Test the string GAUAU.
  z.initZukerAlgorithm("GAUAU");
  ASSERT_EQ(z._seqString, "GAUAU");
  ASSERT_EQ(z._W.size(), 5);
  ASSERT_EQ(z._W[0].size(), 5);
  ASSERT_EQ(z._W[1].size(), 5);
  ASSERT_EQ(z._W[2].size(), 5);
  ASSERT_EQ(z._W[3].size(), 5);
  ASSERT_EQ(z._W[4].size(), 5);

  ASSERT_EQ(z._V.size(), 5);
  ASSERT_EQ(z._V[0].size(), 5);
  ASSERT_EQ(z._V[1].size(), 5);
  ASSERT_EQ(z._V[2].size(), 5);
  ASSERT_EQ(z._V[3].size(), 5);
  ASSERT_EQ(z._V[4].size(), 5);

  ASSERT_EQ(z._W[0][0], 0);
  ASSERT_EQ(z._W[1][1], 0);
  ASSERT_EQ(z._W[2][2], 0);
  ASSERT_EQ(z._W[3][3], 0);
  ASSERT_EQ(z._W[4][4], 0);

  ASSERT_EQ(z._W[1][0], 0);
  ASSERT_EQ(z._W[2][0], 0);
  ASSERT_EQ(z._W[3][0], 0);
  ASSERT_EQ(z._W[4][0], 0);
  ASSERT_EQ(z._W[2][1], 0);
  ASSERT_EQ(z._W[3][1], 0);
  ASSERT_EQ(z._W[3][2], 0);
  ASSERT_EQ(z._W[4][1], 0);
  ASSERT_EQ(z._W[4][2], 0);
  ASSERT_EQ(z._W[4][3], 0);

  ASSERT_EQ(z._V[0][0], 100);
  ASSERT_EQ(z._V[1][1], 100);
  ASSERT_EQ(z._V[2][2], 100);
  ASSERT_EQ(z._V[3][3], 100);
  ASSERT_EQ(z._V[4][4], 100);

  ASSERT_EQ(z._V[1][0], 100);
  ASSERT_EQ(z._V[2][0], 100);
  ASSERT_EQ(z._V[3][0], 100);
  ASSERT_EQ(z._V[4][0], 100);
  ASSERT_EQ(z._V[2][1], 100);
  ASSERT_EQ(z._V[3][1], 100);
  ASSERT_EQ(z._V[3][2], 100);
  ASSERT_EQ(z._V[4][1], 100);
  ASSERT_EQ(z._V[4][2], 100);
  ASSERT_EQ(z._V[4][3], 100);
}


// Test the function V.
TEST(ZukerAlgorithmTest, computeV)
{
  ZukerAlgorithm z;

  z.initZukerAlgorithm("AUCG");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], -1);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(0, 3);
  ASSERT_FLOAT_EQ(z._V[0][3], -1);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], 0);
  z.computeV(1, 3);
  ASSERT_FLOAT_EQ(z._V[1][3], 0);
  z.computeV(2, 3);
  ASSERT_FLOAT_EQ(z._V[2][3], -1);

  // Test the string "AAA".
  z.initZukerAlgorithm("AAA");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], 0);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], 0);

  // Test the string "AU".
  z.initZukerAlgorithm("AU");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], -1);

  // Test the string "ACGU".
  z.initZukerAlgorithm("ACGU");
  z.computeV(0, 1);
  ASSERT_FLOAT_EQ(z._V[0][1], 0);
  z.computeV(0, 2);
  ASSERT_FLOAT_EQ(z._V[0][2], 0);
  z.computeV(1, 2);
  ASSERT_FLOAT_EQ(z._V[1][2], -1);
  z.computeV(1, 3);
  ASSERT_FLOAT_EQ(z._V[1][3], 0);
  z.computeV(2, 3);
  ASSERT_FLOAT_EQ(z._V[2][3], 0);
}


// Test for the method eL.
TEST(ZukerAlgorithmTest, _eL)
{
  ZukerAlgorithm z;

  std::vector<float> resultThree;
  resultThree.push_back(z._eL('A', 'U', 'G', 'C'));
  resultThree.push_back(z._eL('G', 'C', 'A', 'U'));
  resultThree.push_back(z._eL('C', 'G', 'U', 'A'));
  resultThree.push_back(z._eL('C', 'G', 'A', 'U'));
  ASSERT_FLOAT_EQ(resultThree[0], -1);
  ASSERT_FLOAT_EQ(resultThree[1], -1);
  ASSERT_FLOAT_EQ(resultThree[2], -1);
  ASSERT_FLOAT_EQ(resultThree[3], -1);
}

// Test for the function W.
TEST(ZukerAlgorithmTest, computeW)
{
  ZukerAlgorithm z;

  // Test the string "AUCG".
  z.initZukerAlgorithm("AUCG");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], -1);
  ASSERT_FLOAT_EQ(z._W[0][2], -1);
  ASSERT_FLOAT_EQ(z._W[0][3], -2);
  ASSERT_FLOAT_EQ(z._W[1][2], 0);
  ASSERT_FLOAT_EQ(z._W[1][3], -1);
  ASSERT_FLOAT_EQ(z._W[2][3], -1);


  // Test the string "AAA".
  z.initZukerAlgorithm("AAA");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], 0);
  ASSERT_FLOAT_EQ(z._W[0][2], 0);
  ASSERT_FLOAT_EQ(z._W[1][2], 0);

  // Test the string "AU".
  z.initZukerAlgorithm("AUA");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], -1);
  ASSERT_FLOAT_EQ(z._W[0][2], -1);
  ASSERT_FLOAT_EQ(z._W[1][2], -1);

  // Test the string "ACGU".
  z.initZukerAlgorithm("ACGU");
  z.computeW();
  ASSERT_FLOAT_EQ(z._W[0][1], 0);
  ASSERT_FLOAT_EQ(z._W[0][2], -1);
  ASSERT_FLOAT_EQ(z._W[0][3], -2);
  ASSERT_FLOAT_EQ(z._W[1][2], -1);
  ASSERT_FLOAT_EQ(z._W[1][3], -1);
  ASSERT_FLOAT_EQ(z._W[2][3], 0);
}
